<?php
include_once "flowlib.php";

class ControllerExtensionPaymentFlow extends Controller {
	public function index() {
		
		$this->language->load('extension/payment/flow');
		$privData = $this->getData();
		
		$privData['button_confirm'] = $this->language->get('button_confirm');

		$this->load->model('checkout/order');

		$order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);

		if ($order_info) {
			$comercio = $this->config->get('flow_receiverid');
			$ordenCompra = $this->session->data['order_id'];
			$monto = $this->getOrderAmount($order_info);
			$concepto = html_entity_decode('Orden #'.$ordenCompra.' de '.$this->config->get('config_name'), ENT_QUOTES, 'UTF-8');
			$id_producto = $privData['transaction_id'] = $this->session->data['order_id'] . ' - ' . html_entity_decode($order_info['payment_firstname'], ENT_QUOTES, 'UTF-8') . ' ' . html_entity_decode($order_info['payment_lastname'], ENT_QUOTES, 'UTF-8');
			$key = $this->config->get('flow_privatekey');
			
			$tipo_integracion = $this->config->get('flow_skip_type_id');
			$email = $order_info['email'];
			
			$url_exito = $this->url->link('checkout/success');
			$url_fracaso = $this->url->link('checkout/checkout', '', 'SSL');
			$url_confirmacion = $this->url->link('extension/payment/flow/callback', '', 'SSL');
			
			$flow_platform_id = $this->config->get('flow_platform_id');
			$isTestPlatform = !$flow_platform_id || $flow_platform_id == 'test';
			$flow_endpoint = $isTestPlatform?flow_get_endpoint_test():flow_get_endpoint();
			
			$medioPago = 1;
			$privData['flow_action'] = $flow_endpoint;
			$privData['flow_request'] = flow_pack($key, $comercio, 
					$ordenCompra, $monto, $medioPago, $concepto, $id_producto, 
					$url_exito, $url_fracaso, $url_confirmacion,
					$tipo_integracion, $email);

			$templateFile = $this->isVersion1()?"flow.tpl":"flow2.tpl";
			
			if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/extension/payment/' . $templateFile)) {
				$this->template = $this->config->get('config_template') . '/template/extension/payment/' . $templateFile;
			} else {
				if(version_compare(VERSION, '2.2.0.0', '>=')){
					$this->template = 'extension/payment/' . $templateFile;
				} else {
					$this->template = 'default/template/extension/payment/' . $templateFile;
				}
			}
			if ($this->isVersion1()) {
				$this->data = $privData;
				$this->render();
			} else {
				return $this->load->view($this->template, $privData);
			}
		}
	}
	
	private function getOrderAmount($order_info) {
		return $this->currency->format($order_info['total'], $order_info['currency_code'], false, false);
	}
	
	public function callback() {
		$privatekey = $this->config->get('flow_privatekey');
		$comercio = $this->config->get('flow_receiverid');

		$errorResponse = array('status' => 'RECHAZADO', 'c' => $comercio);
		$acceptResponse = array('status' => 'ACEPTADO', 'c' => $comercio);
		
		if (!isset($this->request->post['response'])) {
			$errorResponse['motivo'] = 'Atributo response invalido';
			echo flow_build_response($privatekey, $errorResponse);
			return;
		}
		
		$data = $this->request->post['response'];
		$data = str_replace('&amp;', '&', $data);
		try {
			$params = flow_sign_validate($data);
		} catch (Exception $e) {
			$errorResponse['motivo'] = 'Error interno';
			error_log($e->getMessage());
			echo flow_build_response($privatekey, $errorResponse);
			return;
		}
		
		$error = flow_aget($params, 'error');
		if ($error != null) {
			error_log("Error recibido: $error");
			$acceptResponse['motivo'] = "Error recibido: $error";
			echo flow_build_response($privatekey, $acceptResponse);
			return;
		}
		
		$status = flow_aget($params, 'status');
		if ($status == null) {
			error_log("Peticion inválida (sin status)");
			$errorResponse['motivo'] = "Peticion inválida (sin status)";
			echo flow_build_response($privatekey, $errorResponse);
			return;
		}
		
		$order_id = flow_aget($params, 'kpf_orden');
		$amount   = flow_aget($params, 'kpf_monto');

		$order_status_completed_id = $this->config->get('flow_completed_status_id');
		$order_status_failed_id    = $this->config->get('flow_failed_status_id');
		
		$this->load->model('checkout/order');
		$order_info = $this->model_checkout_order->getOrder($order_id);
		
		if ($order_info) {
			$order_amount_valid = $this->getOrderAmount($order_info) == $amount;
			$order_status_valid = $order_info['order_status_id'] != $order_status_completed_id;
			if ($order_amount_valid) {
				if ($order_status_valid) {
					// ignorar potenciales errores en cambio de estado
					try {
						if ($this->isVersion1()) {
							if ($status == 'EXITO') {
								$this->model_checkout_order->confirm($order_id, $order_status_completed_id);
							} else {
								$this->model_checkout_order->update($order_id, $order_status_failed_id);
							}
						} else {
							if ($status == 'EXITO') {
								$this->model_checkout_order->addOrderHistory($order_id, $order_status_completed_id);
							} else {
								$this->model_checkout_order->addOrderHistory($order_id, $order_status_failed_id);
							}
						}
					} catch (Exception $e) {
						$acceptResponse['warning'] = $e->getMessage();
						error_log($e->getMessage());
					}
					echo flow_build_response($privatekey, $acceptResponse);
					return;
				} else {
					$errorResponse['motivo'] = "Status invalido $order_status_completed_id ".$order_info['order_status_id'];
				}
			} else {
				$errorResponse['motivo'] = "Monto invalido $amount ".$this->getOrderAmount($order_info);
			}
		} else {
			$errorResponse['motivo'] = "Orden $order_id no existe";
		}
		echo flow_build_response($privatekey, $errorResponse);
	}
	
	private function getData() {
		if ($this->isVersion1()) {
			return $this->data;
		}
		return array();
	}
	
	private function isVersion1() {
		return version_compare(VERSION, '2.0.0.0', '<');
	}
}
?>
